from numpy import *
import numpy as np
#from scipy import *

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
# from pylab import *
from shutil import copy
from os import mkdir
import shutil,os,glob
import cPickle as pickle
import xmitgcm
import xarray as xr
import dask.array as da
from progressbar import *
import scipy.interpolate
import sys



runnum = 1
u0 = int(sys.argv[1])
runtype = 'filt'  # 'filt','full','low'
setupname=''
runname='LWRegrid2%s%02dU%02d'%(runtype,runnum,u0)
comments='%s %d topo; 0.1m/s, KL10; '%(runtype,runnum)

# to change U we need to edit external_forcing recompile

outdir0='../results/'+runname+'/'

indir =outdir0+'_Model/indata/'

## Params for below as per Nikurashin and Ferrari 2010b
H = 4000.
U0 = u0/100.
N0 = 1.e-3
f0 = 1.e-4
print N0

# need some info.  This comes from `leewave3d/EmbededRuns.ipynb` on `valdez.seos.uvic.ca`
# the maxx and maxy are for finer scale runs.  
dx0=100.
dy0=100.
maxx = 409600
maxy = 118400


# topography parameters:
useFiltTop=False
useLowTopo=False
useFullTopo=True
if runtype=='filt':
    useFiltTop=True
elif runtype=='low':
    useLowTopo=True

'''
This model is 100 x larger than the "coarse" one.  It also has a time
step 4 x shorter, so it takes 400 x the resources to run.  We are
proposing 512 processors versus 16, so it will run 12.5 times slower.
We did 10 h in 53 minutes for the old runs, so N hours of new runs will take
(N /10)*11 h.  If we do N = 20 h of run time, then that should take 22 h of compute time.  This is about 12,000 h of cpu time.  I have about 60k.  

'''
    
# model size
nx = 32*128
ny = 16*74

nz = 400

def regrid(xh,yh,h,x,y):
    print('Start regrid!')
    d = np.zeros((len(y),len(x)))
    for ii in range(len(x)-1):
        for jj in range(len(y)-1):
            inx = np.where((xh>=x[ii])&(xh<x[ii+1]))[0]
            iny = np.where((yh>=y[jj])&(yh<y[jj+1]))[0]
            if (len(inx)>0) & (len(iny)>0):
                d[jj,ii]=np.mean(h[:,inx][iny,:])
            else:
                d[jj,ii]=d[jj,ii-1]
    print('End regrid!')
    return(d)

def lininc(n,Dx,dx0):
    a=(Dx-n*dx0)*2./n/(n+1)
    print a
    dx = dx0+arange(1.,n+1.,1.)*a
    return dx


#### Set up the output directory
backupmodel=1
if backupmodel:
  try:
    mkdir(outdir0)
  except:
    print outdir0[:-1]
    import datetime
    import time
    ts = time.time()
    st=datetime.datetime.fromtimestamp(ts).strftime('%Y%m%d%H%M%S')
    shutil.move(outdir0[:-1],outdir0[:-1]+'.bak'+st)
    mkdir(outdir0)

    print outdir0+' Exists'

  outdir=outdir0+'_Model/'
  try:
    mkdir(outdir)
  except:
    print outdir+' Exists'
  outdir=outdir+'input/'
  try:
    mkdir(outdir)
  except:
    print outdir+' Exists'
  try:
      mkdir(outdir+'/figs/')
  except:
    pass

  copy('gendata.py',outdir)
else:
  outdir=outdir+'_Model/input/'

## Copy some other files
print "Copying files"

try:
  shutil.rmtree(outdir+'/../code/')
except:
  print "code is not there anyhow"
shutil.copytree('../code', outdir+'/../code/')
shutil.copytree('../python', outdir+'/../python/')

try:
  shutil.rmtree(outdir+'/../build/')
except:
  print "build is not there anyhow"
print outdir+'/../build/'
mkdir(outdir+'/../build/')

# copy any data that is in the local indata
shutil.copytree('../indata/', outdir+'/../indata/')

shutil.copy('../build/mitgcmuvU%02d'%u0, outdir+'/../build/mitgcmuv')
shutil.copy('../build/mitgcmuvU%02d'%u0, outdir+'/../build/mitgcmuvU%02d'%u0)
shutil.copy('../build/Makefile', outdir+'/../build/Makefile')
shutil.copy('data', outdir+'/data')
shutil.copy('data00', outdir)
shutil.copy('data01', outdir)
shutil.copy('data02', outdir)
shutil.copy('eedata', outdir)
shutil.copy('data.kl10', outdir)
try:
  shutil.copy('data.kpp', outdir)
except:
  pass
#shutil.copy('data.rbcs', outdir)
try:
    shutil.copy('data.obcs', outdir)
except:
    pass
try:
  shutil.copy('data.diagnostics', outdir)
except:
  pass
try:
  shutil.copy('data.pkg', outdir+'/data.pkg')
except:
  pass
try:
  shutil.copy('data.rbcs', outdir+'/data.rbcs')
except:
  pass

print "Done copying files"

####### Make the grids #########

# Make grids:

dx0=100
dy0=100


##### Dx ######

dx = np.zeros(nx)+100.

# dx = zeros(nx)+100.
x=np.cumsum(dx)
x=x-x[0]
maxx=np.max(x)
print 'XCoffset=%1.4f'%x[0]

##### Dy ######

dy = np.zeros(ny)+100.

# dx = zeros(nx)+100.
y=np.cumsum(dy)
y=y-y[0]
maxy=np.max(y)
print 'YCoffset=%1.4f'%y[0]

# save dx and dy
with open(indir+"/delX.bin", "wb") as f:
  dx.tofile(f)
f.close()
with open(indir+"/delY.bin", "wb") as f:
  dy.tofile(f)
f.close()
# some plots
fig, ax = plt.subplots(2,1)
ax[0].plot(x/1000.,dx)
ax[1].plot(y/1000.,dy)
#xlim([-50,50])
fig.savefig(outdir+'/figs/dx.pdf')

######## Bathy ############
# get the topo:

if 1:
    if useFiltTop:
        print "Using Filtered topo"
        with open("../results/indata/hband.pickle","rb") as f:
            hh = pickle.load(f)
    elif useLowTopo:
        print "Using Lowpass topo"
        with open("../results/indata/hlow.pickle","rb") as f:
            hh = pickle.load(f)
    else:
        print "Not Using Filtered topo"
        with open("../results/indata/h.pickle","rb") as f:
            hh = pickle.load(f)
            print np.shape(hh['h'])
            #print np.shape(h),np.shape(h0)


    #d = regrid(hh['xh'],hh['yh'],hh['h'],x,y)
    fit=scipy.interpolate.RectBivariateSpline(hh['xh'],hh['yh'],hh['h'].T)
    d = fit(x,y).T 
    d=d-H

    with open(indir+"/topog.bin", "wb") as f:
        d.tofile(f)
        f.close()

    print shape(d)

    fig, ax = plt.subplots(2,1)
    print shape(x),shape(d)
    ax[0].plot(x/1.e3,d[0,:].T)
    pcm=ax[1].pcolormesh(x/1.e3,y/1.e3,d,rasterized=True)
    fig.colorbar(pcm,ax=ax[1])
    fig.savefig(outdir+'/figs/topo.png')


##################
# dz:
# dz is from the surface down (right?).  Its saved as positive.
dz = ones((1,nz))*H/nz

with open(indir+"/delZ.bin", "wb") as f:
	dz.tofile(f)
f.close()
z=np.cumsum(dz)

####################
# temperature profile...
#
# temperature goes on the zc grid:
g=9.8
alpha = 2e-4
T0 = 28+cumsum(N0**2/g/alpha*(-dz))

with open(indir+"/TRef.bin", "wb") as f:
	T0.tofile(f)
f.close()
#plot
plt.clf()
plt.plot(T0,z)
plt.savefig(outdir+'/figs/TO.pdf')

#####################
#
# Temperature initial conditions
#
# These come from the "Coarse" runs...
coarsename = '/scr/jklymak/LWCoarse2/CW3d%s01U%02d/_Model/input/'%(runtype,u0)
print(coarsename)

def interpolateAtDepth(X,T,x0,y0,x,y,defaultval,block_id=None):
    import scipy.interpolate
    good = np.where(T[0,0,:,:]>20.)
    bad = np.where(T[0,0,:,:]<=20.)
    Tt = X.copy()[0,0,:,:]
    #print(good)
    if (len(good[0])>0) & (len(bad[0])>0):
        Tt[bad]= Tt[good].mean()
    else:
        Tt[bad]=defaultval
    if np.shape(X)[-1]>1:
        xout=np.zeros((1,1,ny,nx))   
        fit=scipy.interpolate.RectBivariateSpline(x0,y0,Tt.T)
        xout = fit(x,y).T 
    else:
        xout=np.ones((1,1,1,1))
    print(block_id)
    return xout

class writable_mds_store:
    '''
    This will write each chunk to a binary file.  Really only works if chunks are the k-index of the matrix
    '''
    def __init__(self, prefix, itern, suffix='data', dtype='>f4'):
        self.prefix = prefix
        self.itern = itern
        self.suffix = suffix
        self.dtype = dtype
        self.fname = '%s.%010d.%s' % (self.prefix, self.itern, self.suffix)
        # zero the file:
        with open(self.fname,'wb+') as f:
            pass

    def __setitem__(self, idx, data):
        import os
        tslice = idx[0]
        assert tslice.step is None
        assert (tslice.stop - tslice.start) == 1
        n = tslice.start
        nbytes = data.astype(self.dtype).nbytes
        kslice = idx[1]
        with open(self.fname,'rb+') as f:
            f.seek(nbytes*kslice.start,os.SEEK_SET)
            f.write(data.astype(self.dtype).tobytes())


# one-d field...
todo = 'Eta'
print('Regriding %s'%todo)
data = xmitgcm.open_mdsdataset(data_dir=coarsename,
                               prefix={todo},iters=36000,read_grid=True,
                               geometry='cartesian',endian='<')
# x, y, nx, ny are determined elsewhere, but set the new grid...
fit=scipy.interpolate.RectBivariateSpline(data['XC'].values,data['YC'].values,data[todo].T)
xout = fit(x,y).T
print(np.shape(xout))
with open(indir+'/%sinit.bin'%todo,'wb') as f:
    xout.tofile(f)
#tm.store(tnew_store,lock=True)
#tm.close()
print('Done regriding %s'%todo)

if 1:
    for todo,defaultval in zip(['T','U','V'],[27.5,0.1,0]):
        print('Regriding %s'%todo)
        data = xmitgcm.open_mdsdataset(data_dir=coarsename,
                                       prefix=[todo,'T'],iters=36000,
                                       read_grid=True,geometry='cartesian',endian='<',
                                       chunks={'Z':1,'time':1})
        print(data)
        # x, y, nx, ny are determined elsewhere, but set the new grid...
        tm = da.map_blocks(interpolateAtDepth,
                           data[todo].data,data['T'].data,
                           data['XC'].values,data['YC'].values,
                           x,y,defaultval,
                           chunks=(1,1,ny,nx),
                           dtype=data[todo].data.dtype)

        tnew_store = writable_mds_store(indir+'/%sinit'%todo,0,dtype=tm.dtype,suffix='bin')
        tm.store(tnew_store,lock=True)
        #tm.close()
        print('Done regriding %s'%todo)



########################
# RBCS sponge and forcing
# In data.rbcs, we have set tauRelaxT=17h = 61200 s
# here we wil set the first and last 50 km in *y* to relax at this scale and
# let the rest be free.

# iny = np.where((y<50e3) | (y>maxy-50e3))[0]

# aa = np.zeros((nz,ny,nx))
# for i in iny:
#     aa[:,:,i]=1.

# with open(indir+"/spongeweight.bin", "wb") as f:
#     aa.tofile(f)
# f.close()

# aa=np.zeros((nz,ny,nx))
# aa+=T0[:,newaxis,newaxis]
# print shape(aa)

# with open(indir+"/Tforce.bin", "wb") as f:
#     aa.tofile(f)
# f.close()

###### Manually make the directories
#for aa in range(128):
#    try:
#        mkdir(outdir0+'%04d'%aa)
#    except:
#        pass

print 'Writing info to README'
############ Save to README
with open('README','r') as f:
  data=f.read()
with open('README','w') as f:
  import datetime
  import time
  ts = time.time()
  st=datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H:%M:%S')
  f.write( st+'\n')
  f.write( outdir+'\n')
  f.write(comments+'\n\n')
  f.write(data)

print 'All Done!'

print 'Archiving to home directory'

try:
    shutil.rmtree('../archive/'+runname)
except:
    pass

shutil.copytree(outdir0+'/_Model/input/', '../archive/'+runname+'/input')
shutil.copytree(outdir0+'/_Model/python/', '../archive/'+runname+'/python')
shutil.copytree(outdir0+'/_Model/code', '../archive/'+runname+'/code')

exit()

