# -*- coding: utf-8 -*-
# In[1]:
# This version assumes doubly periodic.  Makes a lot of code easier.
#  - uses correct PE
#  - adds wE term.

#import matplotlib.pyplot as plt
import numpy as np
import pickle
#import matplotlib.colors as mcolors
import xarray as xr
import xmitgcm as xm
import dask.array as da
import sys
from dask.diagnostics import ProgressBar

pbar = ProgressBar(minimum=300.)

print(len(sys.argv))
print(sys.argv)
if len(sys.argv)==5:
    pre = sys.argv[1]
    U0  = float(sys.argv[2])
    first = int(sys.argv[3])
    diter = int(sys.argv[4])
else:
    sys.exit('GetEnergyBudget.py prefix U0 last')

submean = True

N0=1e-3
g=9.8
alpha = 2e-4
dz=10.
T0 = 28-np.cumsum(N0**2/g/alpha*dz*np.ones(400))

dss=[0,0,0]
v0 = np.array([0.,0.,0.])
u0 = np.array([0.,0.,0.])
iters = np.array([-2*diter,-diter,0])+first
print(iters)
for i in range(3):
    n=i
    fname = '../results/%s/ds%010d.nc'%(pre,iters[i])
    print(fname)
    dss[i] = xr.open_dataset(fname, chunks={'Z':10,'Zl':10})
    dss[i]['T0']=(('Z'),T0)
    if submean:
        v0[i] = (dss[i]['V'].isel(Z=slice(50,200)).mean()).values
        u0[i] = (dss[i]['U'].isel(Z=slice(50,200)).mean()).values-U0
    else:
        v0[i]=0.
        u0[i]=0.
    dss[n]['V']=dss[n]['V']-v0[i]
    dss[n]['U']=dss[n]['U']-u0[i]

    print(dss[n]['time'])

energy = [[],[],[]]
for n in range(3):
    energy[n] = xr.Dataset(coords={'Z':dss[n]['Z'].values,
            'time':dss[n]['time'].values,
            'drF':dss[n]['drF'].values,
            'area':dss[n]['rA'].sum().values})

ds=dss[1]

## get dWdP term at each depth.
print('Starting d(WP)/dz')
P = ds['PH'] # m^2/s^2
# This doesn't make any difference.  Why? Because mean w = 0, Dummy.  - (ds['T0'] - energy[1]['Tznew']).cumsum(axis=0)*2.e-4 * g * 10.
PL = (P[:,1:,:,:].data*ds['hFacC'][1:,:,:].data +
        P[:,:-1,:,:].data*ds['hFacC'][:-1,:,:].data)
PL = PL / (ds['hFacC'][1:,:,:].data + ds['hFacC'][:-1,:,:].data )
w = (ds['W'] +
     ds['W'].roll(XC=1).data +
     ds['W'].roll(XC=-1).data +
     ds['W'].roll(YC=1).data +
     ds['W'].roll(YC=-1).data)/5.

WP = np.zeros(len(energy[1]['Z'])+1)
WP[1:-1] = (((w.isel(Zl=slice(1, None)) *
            PL * ds['rA'])).sum(dim=('YC', 'XC')).values)[0]  # m^2 m^3/s^3
dWPz = np.diff(WP)/ds['drF']  # m^2 m^2/s^3
energy[1]['dWPdz'] = -dWPz

print(energy[1].keys())


print('Starting Body Force')
## Get the body force....
f0=1e-4
# clean up...
## sizes wrong here....
# put onto YC:
VC = 0.5*(ds['V'].data + ds['V'].roll(YG=1).data)
xx = (ds['hFacC'] * VC[0] * f0 * U0 * ds['rA']).sum(dim=('XC', 'YC'))
print(xx)
with pbar:
    energy[1]['Bf'] = xx.compute()
print('Done Bf')

## get background T and N2
print('Starting Tz0\n\n')
n=1
with pbar:
    energy[n]['areaZ'] =  (dss[n]['hFacC'] *
            dss[n]['rA']).sum(dim=('YC','XC')).compute()
    # this is the depth mean...
    energy[n]['Tz0'] = (dss[n]['T'].isel(time=0) *
            dss[n]['hFacC'] *
            dss[n]['rA']).sum(dim=('YC','XC')
            ).compute() / energy[n]['areaZ']

print('Starting Tz\n\n')
for n in range(3):
    with pbar:
        t = dss[n]['T']
        t = t.where(t>20.)
        bins = np.linspace(t.min().values, t.max().values, 40000)
        t = dss[n]['T'].data
        # histogram the tempreatures...
        N, bins = da.histogram(t, bins)
        # get a curve of cumulative area as a function of temperature...
        cumareasT = np.cumsum(N.compute())*np.mean(dss[0]['rA'].values)
        Tbins = (bins[1:]+bins[:-1])/2.
        # get a curve of cumulative area as a function of depth
        cumareasZ = np.cumsum(energy[1]['areaZ'].values[::-1])
        Z = dss[n]['Z'][::-1] + 5.
        Z = Z[cumareasZ>1e-10]
        cumareasZ = cumareasZ[cumareasZ>1e-10]
        # get the depth of each temperature bin as a function of cumulative area.
        zTbin = np.interp(cumareasT, cumareasZ,Z)
        # now map back to grid z...  This is the minimum PE state
        Tz = np.interp(energy[n]['Z'], zTbin, Tbins)
        # might as well make a proper array
        energy[n]['Tz'] = Tz + dss[n]['Z']*0

Tz = -np.sort(-energy[1]['Tz'].values)
rhoz = -Tz*2.e-4
drhoz1 = np.diff(rhoz)/10.
N00 = 0.*energy[1]['Z']
N00[1:-1] = 0.5 * (drhoz1[:-1] + drhoz1[1:])
N00[0] = drhoz1[0]
N00[-1] = drhoz1[-1]
N00 = N00 * g

## get the Background PE change...  Could do this on home computer
rho0 = -2e-4*(energy[0]['Tz'])*energy[1]['areaZ']
rho2 = -2e-4*(energy[2]['Tz'])*energy[1]['areaZ']
rho2 = rho2 - rho2.mean() + rho0.mean()
PE0 = (rho0 * g * dss[0]['Z']*dss[0]['drF']).values
PE2 = (rho2 * g * dss[0]['Z']*dss[0]['drF']).values
# these units haven't been introduced: This is mW/m^2 vertical integral averaged
# over the area
energy[1]['dBPEdt']=((PE2.sum()-PE0.sum())/energy[1]['areaZ'][0]/3600.*1e6).compute().data

## KE and PE:
if 1:
    for n in range(3):
        print('Starting KE')
        # this is spatially centered correctly.  It smooths a bit.
        with pbar:
            u = 0.5 * (dss[n]['U'] + dss[n]['U'].roll(XG=1).data)
            xx = (0.5 * (u**2 * dss[n]['hFacW'].data *
                    dss[n]['rA'].data).sum(dim=('YC','XG'))
                    ).compute()
            v = 0.5 * (dss[n]['V'] + dss[n]['V'].roll(YG=1).data)
            xx += (0.5 * (v**2 * dss[n]['hFacS'].data) *
                   dss[n]['rA'].data).sum(dim=('YG','XC')).compute()
            w = 0.5 * (dss[n]['W'] + dss[n]['W'].roll(Zl=1).data)
            xx += (0.5 * (w**2 * dss[n]['hFacS'].data) *
                   dss[n]['rA'].data).sum(dim=('YC','XC')).values
        energy[n]['KE']=xx[0]
        # get background temperature

        with pbar:
            energy[n]['PE'] = ((g * 2e-4 * (dss[n]['T'] - energy[1]['Tz']))**2
                    / 2. / N00 *
                    dss[n]['hFacC'] *
                    dss[n]['rA']).sum(dim=('XC','YC')).isel(time=0).compute()

    print('Done KE and PE')

## dwEdz
print('Start dwEdz')
xx = 0.
for zroll in [0,1]:
    with pbar:
        w = (dss[n]['W'].data +
            dss[n]['W'].roll(XC=1).data +
            dss[n]['W'].roll(XC=-1).data +
            dss[n]['W'].roll(YC=-1).data +
            dss[n]['W'].roll(YC=1).data)/5.
        u = (dss[n]['U'] +
            dss[n]['U'].roll(XG=1).data +
            dss[n]['U'].roll(XG=-1).data +
            dss[n]['U'].roll(YC=1).data +
            dss[n]['U'].roll(YC=-1).data)/5.
        xx = (0.5 * (u**2 *
            dss[n]['rA'].data *
            w)).sum(dim=('XG','YC')
            ).compute()
        v = (dss[n]['V'] +
            dss[n]['V'].roll(YG=1).data +
            dss[n]['V'].roll(YG=-1).data +
            dss[n]['V'].roll(XC=-1).data +
            dss[n]['V'].roll(XC=1).data)/5.
        xx += (0.5 * (v**2 *
                dss[n]['rA'].data *
                w)).sum(dim=('XC','YG')
                ).compute()
        # PE:
        T = dss[n]['T'].values[0]
        nz,ny,nx = np.shape(T)
        for k in range(nz):
            T[k][T[k]==0]=(energy[n]['Tz'].values)[k]
        dss[n]['Tf'] = dss[n]['T']*0. + T
        t = (dss[n]['Tf'] +
            dss[n]['Tf'].roll(XC=1).data +
            dss[n]['Tf'].roll(XC=-1).data +
            dss[n]['Tf'].roll(YC=1).data +
            dss[n]['Tf'].roll(YC=-1).data)/5.

        xx += (((g * 2e-4 * (t - energy[1]['Tz'])
                * dss[n]['hFacC'])**2 /
                2. / N00) *
                dss[n]['rA'] * w).sum(dim=('XC','YC')
                ).compute()

energy[1]['dwEdz'] = 0*energy[1]['Z']
energy[1]['dwEdz'][:-1] = -np.diff(xx[0])/energy[1]['drF'][:-1]

print('Done dwEdz')

## Epsilon:
print('Starting epsilon')
ds['KLepsZ']=xr.DataArray(ds['KLeps'].data,dims=('time','Z','YC','XC'),
                          coords={'Z':ds['Z'].values})

with pbar:
    energy[1]['eps'] = (ds['KLepsZ'] * ds['hFacC'] *
            ds['rA']).sum(dim=('XC', 'YC')).isel(time=0).compute()


# WRITE!
for n in range(3):
    name='EnergyDemean'+pre+'%010dStep%03d.nc'%(iters[1],n)
    energy[n].to_netcdf(path=name, mode='w', format='NETCDF4')
